function JSONParser(str)
{
    this.source = str;
    this.idx = 0;
}

JSONParser.prototype =
{
    parse:function()
    {
        lookahead = this.scanner();

        if (lookahead.type == this.OBJECT_START)
        {
            return this.parseObject();
        }
        else if (lookahead.type == this.ARRAY_START)
        {
            return this.parseArray();
        }
        else
        {
            this.throw_syntax_error("Expected object or array");
        }
    },

    parseObject:function()
    {
        this.match(this.OBJECT_START);

        var object = new Object;

        while (lookahead.type != this.OBJECT_END)
        {
            var name = lookahead.value;

            if (lookahead.type == this.STRING)
            {
                this.match(this.STRING);
            }
            else if (lookahead.type == this.CONST)
            {
                this.match(this.CONST);
            }
            else
            {
                this.throw_syntax_error("Property name at " + this.idx + " must be string or literal");
            }

            this.match(this.PAIR_SEP);

            object[name] = this.parseValue();

            if (lookahead.type != this.OBJECT_END)
            {
                this.match(this.LIST_SEP);
            }
        }
        this.match(this.OBJECT_END);

        return object;
    },

    parseArray:function()
    {
        this.match(this.ARRAY_START);

        var array = new Array;
        var arrIdx = 0;

        while (lookahead.type != this.ARRAY_END)
        {
            array[arrIdx++] = this.parseValue();

            if (lookahead.type != this.ARRAY_END)
            {
                this.match(this.LIST_SEP);
            }
        }
        this.match(this.ARRAY_END);

        return array;
    },

    parseValue:function()
    {
        if (lookahead.type == this.OBJECT_START)
        {
            return this.parseObject();
        }
        else if (lookahead.type == this.ARRAY_START)
        {
            return this.parseArray();
        }
        else if (lookahead.type == this.STRING)
        {
            var value = lookahead.value;
            this.match(this.STRING);
            return value;
        }
        else if (lookahead.type == this.CONST)
        {
            var value = lookahead.value;
            this.match(this.CONST);
            return value;
        }
        else
        {
            this.throw_syntax_error("Unexpected token at " + this.idx);
        }
    },

    scanner:function()
    {
        if (this.idx >= this.source.length)
        {
            return null;
        }

        var lookahead = new Object;

        var state = this.State_NONE;
        var done = false;
        var start;

        while (!done)
        {
            var ch = this.source.charAt(this.idx);

            switch (state)
            {
                case this.State_NONE:
                {
                    if (this.is_white_space(ch))
                    {
                        ++this.idx;
                        continue;
                    }

                    var token = this.is_terminal(ch);
                    if (token != this.NONE)
                    {
                        lookahead.type = token;
                        done = true;
                    }
                    else if (ch == '"')
                    {
                        state = this.State_STRING;
                        start = this.idx + 1;
                    }
                    else
                    {
                        state = this.State_LITERAL;
                        start = this.idx;
                    }
                    break;
                }
                case this.State_STRING:
                {
                    if (ch == '"')
                    {
                        lookahead.type = this.STRING;
                        lookahead.value = this.source.substr(start, this.idx - start);
                        done = true;
                    }
                    break;
                }
                case this.State_LITERAL:
                {
                    if (this.is_white_space(ch) || this.is_terminal(ch))
                    {
                        --this.idx;
                        var value = this.source.substr(start, this.idx - start + 1);
                        if (value == "true")
                        {
                            lookahead.value = true;
                        }
                        else if (value == "false")
                        {
                            lookahead.value = false;
                        }
                        else if (value == "null")
                        {
                            lookahead.value = null;
                        }
                        else
                        {
                            lookahead.value = new Number(value).valueOf();
                        }
                        lookahead.type = this.CONST;
                        done = true;
                    }
                    break;
                }
            }

            ++this.idx;
        }

        return lookahead;
    },

    match:function(type)
    {
        if (type != lookahead.type)
        {
            this.throw_syntax_error("Found " + this.token_names[lookahead.type] + " - expected " + this.token_names[type] + " at " + this.idx);
        }

        lookahead = this.scanner();
    },

    is_terminal:function(ch)
    {
        switch (ch)
        {
            case '{': return this.OBJECT_START;
            case '}': return this.OBJECT_END;
            case ':': return this.PAIR_SEP;
            case '[': return this.ARRAY_START;
            case ']': return this.ARRAY_END;
            case ',': return this.LIST_SEP;
            default: return this.NONE;
        }
    },

    is_white_space:function(ch)
    {
        switch (ch)
        {
            case ' ':
            case '\n':
            case '\t':
            case '\r': return true;
            default: return false;
        }
    },

    throw_syntax_error:function(message)
    {
        message = message + "\n";
        var start = Math.max(0, this.idx - 20);
        var len = Math.min(40, this.source.length - start);
        message = message + this.source.substr(start, len);
        throw new SyntaxError(message);
    },

    State_NONE:0,
    State_STRING:1,
    State_LITERAL:2,

    NONE:0,
    OBJECT_START:1,
    OBJECT_END:2,
    PAIR_SEP:3,
    ARRAY_START:4,
    ARRAY_END:5,
    LIST_SEP:6,
    STRING:7,
    CONST:8,

    token_names:["", "OBJECT_START", "OBJECT_END", "PAIR_SEP" ,"ARRAY_START", "ARRAY_END", "LIST_SEP", "STRING", "CONST"]
}
